package ejb30.interceptors.session.client;

import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.ejb.EJB;

import ejb30.interceptors.session.*;
import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;
// Summary: 
// These test cases essentially checks if interceptors are 
// invoked before every business methods. 
// The test cases also check if application exceptions are 
// propogated correctly back to the client

public class Client {

    private static SimpleReporterAdapter stat = 
        new SimpleReporterAdapter("appserver-sqe");

    private static String testID = "EJB30-Interceptor";

    public static void main (String[] args) {

        if (args.length==1) {
           testID=args[0];
        } 
        System.out.println("testID="+testID);
        stat.addDescription(testID);
        Client client = new Client(args);
        client.doTests();
        // stat.printSummary(testID+"ID");
    }  
    
    public Client (String[] args) {
         String testSuiteID = testID;

    }
    
    private static @EJB Sful sful;

    public void doTests() {
	doTest1();
        /*
	doTest2();
	doTest3();
	doTest4();
	doTest5();
	doTest6();
        doTest9();
        doTest10(); */
	doTest7_8();
    }

    // Each time to invok a business methods in stateful
    // results the interceptorCallCount to increase by 1,
    // which indicates all interceptors for this stateful
    // have been called once.
    private void doTest1() {
        String testcase =  testID + "-CallCount";
        int count1, count2;

        try {
	    // System.out.println("test1:invoking stateful a few times ");
            count1 = sful.getCount();  // the first invoke
            sful.hello();              // the second invoke
            sful.hello2();             // the third invoke
	    count2 = sful.getCount(); // the last invoke
            System.out.println("test1: count1="+count1+", count2="+count2);
            if ((count2 - count1) == 3) {
	      System.out.println("test1:pass");
              stat.addStatus(testcase, stat.PASS);
	    } else {
	      stat.addStatus(testcase, stat.FAIL);
	    }
        } catch(Exception e) {
	  // e.printStackTrace();
          stat.addStatus(testcase, stat.FAIL);
        }
    }

    // AppException
    private void doTest2() {
        String testcase = testID + "-AppException";
        String exp = "XYZ";

        try {
            sful.throwAppException(exp);
            stat.addStatus(testcase, stat.FAIL);
        } catch (AppException appEx) {
		System.out.println("test2: Got expected Exception: " + appEx);
                if (appEx.getMessage().equals(exp)) {
		  stat.addStatus(testcase, stat.PASS);
		} else {
		  stat.addStatus(testcase, stat.FAIL);
		}
        } catch(Exception e) {
            e.printStackTrace();
            stat.addStatus("testcase" , stat.FAIL);
        }
    }

    //  computeMid(min,max), min < max --> NoSwapArgumentsException
    private void doTest3() {
        String testcase = testID + "-NoSwapArgurmentsException";
        try {
            String result = sful.computeMid(4, 10);
	    System.out.println("test3: Got: " + result);
            stat.addStatus(testcase, stat.PASS);
        } catch (SwapArgumentsException swapEx) {
	    System.out.println("test3: Got unexpected Exception: " + swapEx);
            stat.addStatus(testcase, stat.FAIL);
        } catch(Exception e) {
            e.printStackTrace();
            stat.addStatus(testcase, stat.FAIL);
        }
    }

    //  computeMid(min,max), min > max --> SwapArgumentsException
    private void doTest4() {
        String testcase = testID + "-SwapArgurmentsException";
        try {
            String result = sful.computeMid(23, 10);
	    System.out.println("test4: Got: " + result
			    + " INSTEAD of SwapArgumentsException");
            stat.addStatus(testcase, stat.FAIL);
        } catch (SwapArgumentsException swapEx) {
	    System.out.println("test4: Got expected Exception: " + swapEx);
            stat.addStatus(testcase, stat.PASS);
        } catch(Exception e) {
            e.printStackTrace();
            stat.addStatus(testcase, stat.FAIL);
        }
    }

    // CallBlockedException
    // SfulEJB -- callDummy throws Exception -- dummy.dummy()
    // DummyEJB -- dummy throws CallBlockException, which
    // doesn't handle exception
    // SfulEJB invokes a method, which encounteres an exception, 
    //  but does not handle it.
    private void doTest5() {
        String testcase = testID + "-CallBlockedException";
        try {
            String result = sful.callDummy();
	    System.out.println("test5: Got: " + result
			    + " INSTEAD of CallBlockedException");
            stat.addStatus(testcase , stat.FAIL);
        } catch (CallBlockedException callBlkEx) {
	    System.out.println("test5: Got expected Exception: " + callBlkEx);
            stat.addStatus(testcase , stat.PASS);
        } catch(Exception e) {
            e.printStackTrace();
            stat.addStatus(testcase , stat.FAIL);
        }
    }

    // SfulEJB -- eatException throw EatException -- throw new EatException()
    // SfulEJB -- interceptCall -- catch(EatException ee) return "ate";
    // SfulEJB invokes a method, which encounteres an exception, 
    // and interceptor handles this exception.
    private void doTest6() {
        String testcase = testID + "-EatException";
        try {
            sful.eatException();
	    System.out.println("test6: eatException, pass ");
            stat.addStatus(testcase , stat.PASS);
        } catch(Exception e) {
	    System.out.println("test6: Got unexpected Exception: " + e);
            e.printStackTrace();
            stat.addStatus(testcase , stat.FAIL);
        }
    }

    private void doTest7_8() {
        String testcase = testID + "-PassivateActivate";
        boolean statePassivateActivate = false;
        boolean stateRestored = true;
        System.out.println("test7_8");
        try {
	    // previous resource injection for 1 bean
	    // jndi look up for 19 beans.
	    // The stateful beans will be passivated due to the size
	    //  e.g.    sun-ejb-jar.xml       
   	    //  <max-cache-size>4</max-cache-size>
	    //  <resize-quantity>2</resize-quantity>
	    int sz = 20;
	    Context ctx = new InitialContext();
	    sful.resetLifecycleCallbackCounters();
	    Sful[] sfuls = new Sful[sz];
	    for (int i=0; i<sz; i++) {
	      if (i == 0 ) {
		sfuls[0] = sful;
	      } else { 
    		sfuls[i] = (Sful) 
		  ctx.lookup("ejb30.interceptors.session.Sful");
	      }
              // false -- different beans
    	      int prevIndex = (i ==0) ? 0 : i-1;
    	      System.out.println("Created sful["+i+"], not new: "
    	             + sfuls[prevIndex].equals(sfuls[i]));
	      sfuls[i].setID(i);
	      sfuls[i].getCount();
	    }

	    sleepFor(30);

            // access bean so that the beans are activated.
	    int passivationCount = 0;
	    int activationCount = 0;
            System.out.println("test7, access bean");
	    for (int i=0; i<sz; i++) {
	        sfuls[i].getCount();  // ex when security manager on
		passivationCount = sful.getPrePassivateCallbackCount();
                activationCount = sfuls[i].getPostActivateCallbackCount();
                System.out.println("test7: i="+i+", p="+passivationCount+", a="+activationCount);
	    }

	    // When passivated by timeout, passivationCount == sz,
	    // When passivated by size, passivationCount > sz since
	    // some beans may be passivated twice.
	    // activationCount = sz 

	    // Passivation and Activation happen on all beans
	    if (activationCount >= sz && passivationCount >= sz) {
	      System.out.println("Passivation and Activation happen on all beans");          statePassivateActivate = true;
	    } 

	    // call LifecycleCallbackInterceptor interceptor
	    // interceptor.checkInterceptorID(id)
	    
	    for (int i=0; i<sz; i++) {
	      boolean ok = sfuls[i].isStateRestored();
	      System.out.println("test8:stateRestored[" + i + "]:" + ok);
	      if (!ok) {
		stateRestored = false;
		break;
	      }
	    }
	    if (stateRestored && statePassivateActivate ){
              System.out.println("test8:pass");
	      stat.addStatus(testcase , stat.PASS);
	    } else {
	      stat.addStatus(testcase , stat.FAIL);
	    }
	} catch(Exception e) {
	  System.out.println("test8:Got unexpected Exception");
	  // e.printStackTrace();
	  stat.addStatus(testcase , stat.FAIL);
	}
    }

    private void doTest9() {
        String testcase = testID + "-SetParams";
        boolean checkSetParamsStatus = true;
        Map<String, Boolean> map = sful.checkSetParams();
        try { 
	  for (String testName : map.keySet()) {
            // System.out.println("test9[" + testName + "]: " + map.get(testName));
            checkSetParamsStatus = checkSetParamsStatus && map.get(testName);
	  }
	  System.out.println("test9:checkSetParamsStatus="+checkSetParamsStatus);
          stat.addStatus(testcase , 
                (checkSetParamsStatus == true) ? stat.PASS : stat.FAIL);
	} catch (Exception e) {
	  System.out.println("test9:Got unexpected Exception");
	  // e.printStackTrace();
	  stat.addStatus(testcase , stat.FAIL);
	}
    }

    // AssertionFailedException
    private void doTest10() {
        String testcase = testID + "-AssertionFailedException";
        boolean trueStatus = false;
        try {
            sful.assertIfTrue(true);
        } catch (AssertionFailedException aEx) {
            trueStatus = true;
        } catch (Exception aEx) {
            trueStatus = false;
        }

        boolean failedCountStatus = false;
        int assertFailedCount = 0;
        try {
            failedCountStatus = (sful.getAssertionFailedCount() == 1);
        } catch (Exception aEx) {
            failedCountStatus = false;
        }
        System.out.println("test10:"+trueStatus+", "+sful.getAssertionFailedCount());
        stat.addStatus(testcase, 
	 ((trueStatus && failedCountStatus) == true) ? stat.PASS : stat.FAIL);
    }

    private static void sleepFor(int seconds) {
	while (seconds-- > 0) {
	    try {
	        // System.out.println("" + seconds + " left...");
		Thread.currentThread().sleep(1000);
	    } catch (InterruptedException inEx) {
	    }
	}
    }

}

