package ejb30.interceptors.session;

import javax.interceptor.InvocationContext;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.PrePassivate;
import javax.ejb.PostActivate;
import javax.ejb.EJB;
import javax.annotation.Resource;
import javax.ejb.SessionContext;

public class LifecycleCallbackInterceptor {

    private static int postConstructCallbackCount = 0;
    private static int preDestroyCallbackCount = 0;
    private static int prePassivateCallbackCount = 0;
    private static int postActivateCallbackCount = 0;
    
    private int interceptorID;
    private int computedInterceptorID;

    @EJB Sless sless;
    @Resource SessionContext sessionCtx;

    @PostConstruct
    void postConstruct(InvocationContext ctx) {
	postConstructCallbackCount++;
        System.out.println("**LifecycleCallBack:@PostConstruct, count="+postConstructCallbackCount);
    }

    @PreDestroy
    void preDestroy(InvocationContext ctx) {
	preDestroyCallbackCount++;
        System.out.println("**LifecycleCallBack:@PreDestroy, count="+preDestroyCallbackCount);
    }

    @PrePassivate
    void prePassivate(InvocationContext ctx) {
	prePassivateCallbackCount++;
        System.out.println("**LifecycleCallBack:@PrePassivate, count="+prePassivateCallbackCount);
    }

    @PostActivate
    private void postActivate(InvocationContext ctx) {
	postActivateCallbackCount++;
        System.out.println("**LifecycleCallBack:@PostActivate, count="+postActivateCallbackCount);
    }

    @AroundInvoke
    Object aroundInvoke(InvocationContext ctx)
	        throws Exception {
       // System.out.println("****LifecycleCallBackInterceptor:aroundInvoke by @AroundInvoke*****");
        if (ctx.getMethod().getName().equals("setID")) {
            ctx.getContextData().put("LifecycleCallbackInterceptor", this);
            System.out.println("calling sless from interceptor. sless says " +
                               sless.sayHello());
            System.out.println("invoked business interface = " +
                               sessionCtx.getInvokedBusinessInterface());
        }
        return ctx.proceed();
    }

    public static void resetLifecycleCallbackCounters() {
	prePassivateCallbackCount = 0;
	postActivateCallbackCount = 0;
    }

    public static int getPrePassivateCallbackCount() {
	return prePassivateCallbackCount;
    }

    public static int getPostActivateCallbackCount() {
	return postActivateCallbackCount;
    }
    
    public static int getPostConstructCallbackCount() {
	return postConstructCallbackCount;
    }

    public static int getPreDestroyCallbackCount() {
	return preDestroyCallbackCount;
    }


    void setInterceptorID(int val) {
        this.interceptorID = val;
        this.computedInterceptorID = 2 * val + 1;
    }
    
    int getInterceptorID() {
        return interceptorID;
    }
    
    boolean checkInterceptorID(int val) {
      // System.out.println("**LifecycleCBI:checkInterceptorID("+val+")");
      // System.out.println("**interceptorID="+interceptorID);
      // System.out.println("**computedInterceptorID="+computedInterceptorID);
        return (val == interceptorID) && 
            (computedInterceptorID == 2 * val + 1);
    }
   
}
